| Copyright | (c) Erich Gut |
|---|---|
| License | BSD3 |
| Maintainer | zerich.gut@gmail.com |
| Safe Haskell | None |
| Language | Haskell2010 |
OAlg.AbelianGroup.Liftable
Contents
Description
lifting of abelian homomorphisms.
Synopsis
- zMatrixLift :: Matrix Z -> Matrix Z -> Maybe (Matrix Z)
- prpMatrixZJustLiftable :: XOrtSite 'To (Matrix Z) -> Statement
- prpMatrixZMaybeLiftable :: X Z -> Statement
- prpMatrixZLiftable :: Statement
- xLiftable :: Multiplicative c => XOrtSite 'To c -> X (c, c)
Lifting
zMatrixLift :: Matrix Z -> Matrix Z -> Maybe (Matrix Z) Source #
tries to solve the equation a .* x == y
Property Let a and y be in , then holds:Matrix Z
- If
is not equal toendythen evaluatingendawill end up in azMatrixLifta yNotLiftable-exception. - If
is equal toendyand there exists anendaxinsuch thatMatrixZathen the result of*x==yiszMatrixLifta yotherwise it will beJustxNothing. If there exists a non trivial solution, thenxwill also be non trival.
Proposition
prpMatrixZJustLiftable :: XOrtSite 'To (Matrix Z) -> Statement Source #
validity of zMatrixLift for liftable samples.
prpMatrixZMaybeLiftable :: X Z -> Statement Source #
validity of zMatrixLift where liftable and unliftable samples are validated.
prpMatrixZLiftable :: Statement Source #
validity of zMatrixLift.